/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.startstopp.prozessvew;

import static org.junit.Assert.*;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.sys.startstopp.skriptvew.GlobaleDaten;
import de.bsvrz.sys.startstopp.skriptvew.Kernsystem;
import de.bsvrz.sys.startstopp.skriptvew.StartStoppApp;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_AUSGABE;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_NEUSTART;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_STARTART;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_START_FEHLER_VERHALTEN;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_STOPP_FEHLER_VERHALTEN;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_WARTEART;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_ZUSTAND;

/**
 * Klasse mit Testmethoden der Klasse Inkarnation
 * @author Dambach Werke GmbH
 *
 */
public class InkarnationTest
{
  /**
   * Name der Inkarnation
   */
  String m_name              = "abc";
  
  /**
   * Ausfhrbare Datei der Inakrantion
   */
  String m_ausfuehrbareDatei = "C:/Windows/System32/calc.exe";
  
  /**
   * Aufrufparameter (Anzahl)
   */  
  int m_anzAufrufParameter = 10;
  
  /**
   * Aufrufparameter
   */  
  String m_aufrufParameter   = "p1 p2 p3 &quot;Ein Text&quot; 1234";
  
  /**
   * Startart
   */
  E_STARTART m_option        = E_STARTART.AUTOMATISCH;
  
  /**
   * Neustart
   */
  E_NEUSTART m_neustart      = E_NEUSTART.Ja;
  
  /**
   * Intervall
   */
  String m_intervall         = "Intervall 1";

  /**
   * Anzahl Vorgnger
   */  
  int m_anzVorgaengerBedingungen = 20;
  
  /**
   * Vorgnger
   */
  String m_vorgaenger          = "Vorgaenger_";
  
  /**
   * Warteart
   */
  E_WARTEART m_warteart        = E_WARTEART.BEGINN;
  
  /**
   * Rechner Vorgnger
   */
  String m_rechnerVorgaenger   = "Vorgaengerrechner_";
  
  /**
   * Wartezeit Vorgnger
   */
  long   m_wartezeitVorgaenger = 0;
  
  /**
   * Anazhl Nachfolger
   */
 
  int m_anzNachfolgerBedingungen = 20;
  
  /**
   * Nachfolger
   */
  String m_nachfolger          = "Nachfolger_";
  
  /**
   * Rechner Nachfolger
   */
  String m_rechnerNachfolger   = "Nachfolgerrechner_";
  
  /**
   * Wartezeit Nachfolger
   */
  long   m_wartezeitNachfolger =  0;
  
  /**
   * Standardausgabe Datei
   */  
  String m_standardDatei = "Standardausgabedatei";
  
  /**
   * Standardausgabe Art
   */
  E_AUSGABE m_standardAusgabe = E_AUSGABE.EIGENEDATEI;
  
  /**
   * StartStopp Block ID
   */
  String m_startStoppBlockId  = "0815";
  
  /**
   * Standardfehlerausgabe Datei
   */  
  String m_fehlerDatei = "Standardfehlerdatei";
  
  /**
   * Standardfehlerausgabe Art
   */  
  E_AUSGABE m_fehlerAusgabe = E_AUSGABE.GEMEINSAMEDATEI;

  /**
   * Startfehlerverhalten
   */
  E_START_FEHLER_VERHALTEN m_startFehlerVerhalten = E_START_FEHLER_VERHALTEN.ABBRUCH;
    
  /**
   * Anzahl Startfehler
   */  
  long m_anzahlStartFehler = 123; 

  /**
   * Stoppfehlerverhalten
   */
  E_STOPP_FEHLER_VERHALTEN m_stoppFehlerVerhalten = E_STOPP_FEHLER_VERHALTEN.IGNORIEREN;

  /**
   * Anzahl Stoppfehler
   */  
  long m_anzahlStoppFehler = 321; 

  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    // Testumgebung initialisieren
    
    StartStoppApp a = new StartStoppApp ();
    
    a.setStartStoppKonfiguration( "C:\\JUnit");
    a.setStartStoppInkarnationsName( "StartStopp" );
    a.bestimmeRechnerAdresse();
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen des Inkarnationsnamens
   */
  @Test
  public void testInkarnationName()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    assertEquals( m_name, prozessDatenAusInkarnation.getName()) ;
  }
    
  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen der ausfhrbaren Datei
   */
  @Test
  public void testInkarnationAusfhrbareDatei()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    assertEquals( m_ausfuehrbareDatei, prozessDatenAusInkarnation.getAusfuehrbareDatei()) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen der Aufrufparameter
   */
  @Test
  public void testInkarnationAufrufParameter()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean aufrufParameterOk = true;
    
    List<String> p = prozessDatenAusInkarnation.getAufrufParameter();
    if (p.size() == m_anzAufrufParameter)
    {
      for (int i=0; i<p.size(); ++i)
      {
        String p1 = p.get( i );
        
        if (!(p1.equals(  m_aufrufParameter + " Aufrufparamerer_" + i )))
          aufrufParameterOk = false;
      }
    }
    else
      aufrufParameterOk = false;

    assertEquals( true, aufrufParameterOk) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen der Startart
   */
  @Test
  public void testInkarnationStartArt()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean startArtOk = prozessDatenAusInkarnation.getStartArt().getOption()   == m_option   &&
                         prozessDatenAusInkarnation.getStartArt().getNeuStart() == m_neustart &&
                         prozessDatenAusInkarnation.getStartArt().getIntervallZeit().equals( m_intervall);
                         
    assertEquals( true, startArtOk) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen der Startbedingungen
   */
  @Test
  public void testInkarnationStartBedingung()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean startBedingungOk = true;
    
    List<StartBedingung> b = prozessDatenAusInkarnation.getStartBedingung();
    if (b.size() == m_anzVorgaengerBedingungen)
    {
      for (int i=0; i<b.size(); ++i)
      {
        StartBedingung b1 = b.get( i );
        if (!(b1.getProzessName().equals(  m_vorgaenger + i)) ||
            !(b1.getWarteArt() == m_warteart) ||
            !(b1.getRechnerAlias().equals( m_rechnerVorgaenger + i )) ||
            !(b1.getWarteZeit() == m_wartezeitVorgaenger + i))
          startBedingungOk = false;
      }
    }
    else
      startBedingungOk = false;

    assertEquals( true, startBedingungOk) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen der Stoppbedingungen
   */
  @Test
  public void testInkarnationStoppBedingung()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean stoppBedingungOk = true;
    
    List<StoppBedingung> sb = prozessDatenAusInkarnation.getStoppBedingung();
    if (sb.size() == m_anzVorgaengerBedingungen)
    {
      for (int i=0; i<sb.size(); ++i)
      {
        StoppBedingung b1 = sb.get( i );
        if (!(b1.getProzessName().equals(  m_nachfolger + i)) ||
            !(b1.getRechnerAlias().equals( m_rechnerNachfolger + i )) ||
            !(b1.getWarteZeit() == m_wartezeitNachfolger + i))
          stoppBedingungOk = false;
      }
    }
    else
      stoppBedingungOk = false;

    assertEquals( true, stoppBedingungOk) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen der Standardausgabe
   */
  @Test
  public void testInkarnationStandardAusgabe()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean standardAusgabeOk = true;
    
    StandardAusgabe sa = prozessDatenAusInkarnation.getStandardAusgabe();
    if (!sa.getDateiAlias().equals( m_standardDatei ) ||
        !sa.getStartStoppBlockId().equals( m_startStoppBlockId ) ||
        !(sa.getOption() == m_standardAusgabe))
       standardAusgabeOk = false;

    assertEquals( true, standardAusgabeOk) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen der Standardfehlerausgabe
   */
  @Test
  public void testInkarnationStandardFehlerAusgabe()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean standardFehlerAusgabeOk = true;
    
    FehlerAusgabe fa = prozessDatenAusInkarnation.getStandardFehlerAusgabe();
    
    if (!fa.getDateiAlias().equals( m_fehlerDatei ) ||
        !fa.getStartStoppBlockId().equals( m_startStoppBlockId ) ||
        !(fa.getOption() == m_fehlerAusgabe))
      standardFehlerAusgabeOk = false;
    
    assertEquals( true, standardFehlerAusgabeOk) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen des Startfehlerverhaltens
   */
  @Test
  public void testInkarnationStartFehlerVerhalten ()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean startFehlerverhaltenOk = true;
    
    StartFehlerVerhalten sfv = prozessDatenAusInkarnation.getStartVerhaltenFehler();
    
    if ((sfv.getOption() != m_startFehlerVerhalten) ||
        (sfv.getWiederholungen() != m_anzahlStartFehler))
      startFehlerverhaltenOk = false;
    
    assertEquals( true, startFehlerverhaltenOk) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#Inkarnation(java.lang.String)}.
   * Methode testet das Speichern/Auslesen des Stoppfehlerverhaltens
   */
  @Test
  public void testInkarnationStoppFehlerVerhalten ()
  {
    init ();

    Inkarnation x = new Inkarnation (m_name);
    ProzessDaten prozessDatenAusInkarnation = x.getPd();

    boolean stoppFehlerverhaltenOk = true;
    
    StoppFehlerVerhalten sfv = prozessDatenAusInkarnation.getStoppVerhaltenFehler();
    
    if ((sfv.getOption() != m_stoppFehlerVerhalten) ||
        (sfv.getWiederholungen() != m_anzahlStoppFehler))
      stoppFehlerverhaltenOk = false;
    
    assertEquals( true, stoppFehlerverhaltenOk) ;
  }

  /**
   * Testmethode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#starteInkarnation(long, long)}
   * und {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#stoppeInkarnation(long, boolean)}.
   * Methode startet eine Inkarnation. Dabei wird geprft ob die Inkarnation folgende Zustnde annimmt:<br>
   * - angelegt<br>
   * - gestartet<br>
   * - gestoppt<br>
   */
  @Test
  public void testStarteStoppeInkarnation()
  {
    boolean ok = true;
    
    init ();

    // Wichtig: die Inkarnation muss dem Kernsystem zugeordnet werden, sonst wird auf den
    //          Start des Kernsystems gewartet
    
    Kernsystem ks = new Kernsystem (m_name, 0);
    GlobaleDaten.getInstanz().addKernSystem( ks );

    // Anlegen der Inkarnation
    
    Inkarnation x = new Inkarnation (m_name);
    
    // Prfen ob Inkarnation den Zustand angelegt hat
    
    E_ZUSTAND aktZustand = x.getPd().getZustand();
    ok = (aktZustand == E_ZUSTAND.ANGELEGT);
    if (!ok)
      fail( "Inkarnation nicht im Zustand " + E_ZUSTAND.ANGELEGT.getText() );
    
    // Starten der Inkarnation
    
    if (ok)
    {
      x.starteInkarnation ( 1234, m_anzahlStartFehler );
    
      mySleep (3000);
      
      aktZustand = x.getPd().getZustand();
      ok = (aktZustand == E_ZUSTAND.GESTARTET);

      if (!ok)
        fail( "Inkarnation nicht im Zustand " + E_ZUSTAND.GESTARTET.getText() );
    }

    // Stoppen der Inkarnation
    
    if (ok)
    {
      x.stoppeInkarnation( 1234, true );

      mySleep (3000);
      
      aktZustand = x.getPd().getZustand();
      ok = (aktZustand == E_ZUSTAND.GESTOPPT);
      
      if (!ok)
        fail( "Inkarnation nicht im Zustand " + E_ZUSTAND.GESTOPPT.getText() );
    }

    assertEquals( true, ok) ;
  }

  /**
   * Testmethode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#starteInkarnationZyklisch(long)}.
   * Dieser Test prft das zyklische Starten der Inkarnation. Dieser Test kann je nach eingestellter
   * Zykluszeit sehr lange dauern !!! Mit der jetzigen Einstellung (Start alle 2 Minuten, 3 mal prfen) 
   * dauert der Test bis zu 7 Minuten.
   */
//  @Test
  public void testStarteInkarnationZyklisch()
  {
    boolean ok = true;
    
    init ();

    // Wichtig: die Inkarnation muss dem Kernsystem zugeordnet werden, sonst wird auf den
    //          Start des Kernsystems gewartet
    
    Kernsystem ks = new Kernsystem (m_name, 0);
    GlobaleDaten.getInstanz().addKernSystem( ks );

    // Anlegen der Inkarnation
    
    Inkarnation x = new Inkarnation (m_name);

    StartArt startArt = x.getPd().getStartArt();
    StartFehlerVerhalten startfehlerVerhalten = x.getPd().getStartVerhaltenFehler();
    
    // Startzeit immer zum 2 Minuten Intervall
    
    long delta       = 2;  // Differenz der zyklischen Starts in Minuten
    
    String minute    = "0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,32,34,36,38,40,42,44,46,48,50,52,54,56,58";
    String stunde    = "*";
    String tag       = "*";
    String monat     = "*";
    String wochentag = "*";
    String intervall = minute + " " + stunde + " " + tag + " " + monat + " " + wochentag;
    
    startArt.setIntervallZeit( intervall );
    startArt.setOption( E_STARTART.INTERVALL );
    startfehlerVerhalten.setOption( E_START_FEHLER_VERHALTEN.IGNORIEREN );
    
    // Prfen ob Inkarnation den Zustand angelegt hat
    
    E_ZUSTAND aktZustand = x.getPd().getZustand();
    ok = (aktZustand == E_ZUSTAND.ANGELEGT);
    if (!ok)
      fail( "Inkarnation nicht im Zustand " + E_ZUSTAND.ANGELEGT.getText() );
    
    // Starten der Inkarnation
    
    x.starteInkarnationZyklisch( 1234 );

    // Anzahl wie oft das zyklische Starten geprft werden soll
    
    int anzVersuche = 2;
    for (int versuche = 0; versuche < anzVersuche; ++versuche)
    {
      System.out.println("Versuch = " + versuche);
      // Max. delta + 1 Minuten auf 1. Start warten
      
      int t = 0;
      while (t < (delta + 1) * 60000)
      {
        aktZustand = x.getPd().getZustand();
        if (aktZustand == E_ZUSTAND.GESTARTET)
          break;
        
        t++;
        
        mySleep (1000);
      }
  
      // Prfen ob Inkarnation gestartet wurde
      
      ok =(x.getPd().getZustand() == E_ZUSTAND.GESTARTET);
      if (!ok)
        fail( "Inkarnation nicht im Zustand " + E_ZUSTAND.GESTARTET.getText() );
      
      // Prfen ob Inkarnation auch zum richtigen Zeitpunkt gestartet wurde
      
      int aktMinute = Calendar.getInstance().get( Calendar.MINUTE);
      if ( (aktMinute % delta) != 0 ) 
        fail( "Inkarnation wurde in falscher Minute gestartet " + aktMinute );
      
      // Prozess direkt beenden
      // Wichtig: nicht beendeProzess benutzen
      
      mySleep (3000);  // wichtig damit Inkarnation erkennt, dass Prozess gestartet wurde
      
      Process prozess = x.getProzess();
      prozess.destroy();
      
      mySleep (3000);
  
      ok =(x.getPd().getZustand() == E_ZUSTAND.GESTOPPT);
      if (!ok)
        fail( "Inkarnation nicht im Zustand " + E_ZUSTAND.GESTOPPT.getText() );
    }

    assertEquals( true, ok) ;
  }
  
  /**
   * Testmethode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#isStartBedingungErfuellt ()}.
   */
  @Test
  public void testIsStartBedingungErfuellt ()
  {
    ProzessVerwaltung pv = ProzessVerwaltung.getInstanz();

    String prozess1 = "Prozess 1";
    ProzessDaten pd1 = new ProzessDaten();
    pv.setProzessDaten( prozess1, pd1 );
    pd1.setName( prozess1 );
    pd1.setLetzterStart( new Date ());

    String prozess2 = "Prozess 2";
    ProzessDaten pd2 = new ProzessDaten();
    pv.setProzessDaten( prozess2, pd2 );
    pd2.setName( prozess2 );
    pd2.setLetzterStart( new Date ());

    ProzessDaten prozessDaten = new ProzessDaten ();
    pv.setProzessDaten( m_name, prozessDaten );

    prozessDaten.setName( m_name );
    prozessDaten.setAusfuehrbareDatei( m_ausfuehrbareDatei );
    
    // Startbedingungen definieren
    
    prozessDaten.addStartBedingung   ( prozess1, E_WARTEART.BEGINN.getText(),          "", 0); 
    prozessDaten.addStartBedingung   ( prozess2, E_WARTEART.INITIALISIERUNG.getText(), "", 0);

    // Anlegen der Inkarnation
    
    Inkarnation x = new Inkarnation (m_name);

    // Prozessdaten dieser Startprozesse laden und entsprechend initialisieren
    
    // Prffall 1
    
    pd1.setZustand( E_ZUSTAND.ANGELEGT );
    pd2.setZustand( E_ZUSTAND.ANGELEGT );
    
    boolean erfuellt = x.isStartBedingungErfuellt();
    
    if (erfuellt)
      fail ("Prffall 1: Startbedingung flschlicherweise erfllt !");

    // Prffall 2
    
    pd1.setZustand( E_ZUSTAND.GESTARTET );
    pd2.setZustand( E_ZUSTAND.ANGELEGT );
    
    erfuellt = x.isStartBedingungErfuellt();
    
    if (erfuellt)
      fail ("Prffall 2: Startbedingung flschlicherweise erfllt !");

    // Prffall 3
    
    pd1.setZustand( E_ZUSTAND.ANGELEGT );
    pd2.setZustand( E_ZUSTAND.INITIALISIERT );
    
    erfuellt = x.isStartBedingungErfuellt();
    
    if (erfuellt)
      fail ("Prffall 3: Startbedingung flschlicherweise erfllt !");
    
    // Prffall 4
    
    pd1.setZustand( E_ZUSTAND.GESTARTET );
    pd2.setZustand( E_ZUSTAND.INITIALISIERT );
    
    erfuellt = x.isStartBedingungErfuellt();
    
    if (!erfuellt)
      fail ("Prffall 4: Startbedingung flschlicherweise nicht erfllt !");

    // Prffall 5

    pd1.setZustand( E_ZUSTAND.ANGELEGT );
    pd2.setZustand( E_ZUSTAND.ANGELEGT );

    long wartezeit = 10;
    
    prozessDaten.addStartBedingung   ( prozess1, E_WARTEART.BEGINN.getText(),          "", wartezeit); 
    prozessDaten.addStartBedingung   ( prozess2, E_WARTEART.INITIALISIERUNG.getText(), "", wartezeit);
    
    Date start = new Date ();
    
    pd1.setZustand( E_ZUSTAND.GESTARTET );
    pd2.setZustand( E_ZUSTAND.INITIALISIERT );

    int t = 0;
    while (t < wartezeit + 10)
    {
      if (x.isStartBedingungErfuellt())
        break;
      
      mySleep( 1000 );
    }

    erfuellt = false;
    
    if (x.isStartBedingungErfuellt())
    {
      Date ende = new Date ();
      long delta = (ende.getTime() - start.getTime()) / 1000;
      
      if ((delta < wartezeit) || (delta > wartezeit + 5)) // Kleiner Spielraum nach oben
        fail ("Prffall 5: Startbedingung zeitlich nicht korrekt erfllt: " + delta + " Sekunden");
      
      erfuellt = true;
    }
    
    assertEquals( true, erfuellt) ;
  }

  /**
   * Testmethode fr {@link de.bsvrz.sys.startstopp.prozessvew.Inkarnation#isStoppBedingungErfuellt ()}.
   */
  @Test
  public void testIsStoppBedingungErfuellt ()
  {
    ProzessVerwaltung pv = ProzessVerwaltung.getInstanz();

    String prozess1 = "Prozess 1";
    ProzessDaten pd1 = new ProzessDaten();
    pv.setProzessDaten( prozess1, pd1 );
    pd1.setName( prozess1 );
    pd1.setLetzterStart( new Date ());

    String prozess2 = "Prozess 2";
    ProzessDaten pd2 = new ProzessDaten();
    pv.setProzessDaten( prozess2, pd2 );
    pd2.setName( prozess2 );
    pd2.setLetzterStart( new Date ());

    ProzessDaten prozessDaten = new ProzessDaten ();
    pv.setProzessDaten( m_name, prozessDaten );

    prozessDaten.setName( m_name );
    prozessDaten.setAusfuehrbareDatei( m_ausfuehrbareDatei );
    
    // Startbedingungen definieren
    
    prozessDaten.addStoppBedingung   ( prozess1, "", 0); 
    prozessDaten.addStoppBedingung   ( prozess2, "", 0);

    // Anlegen der Inkarnation
    
    Inkarnation x = new Inkarnation (m_name);

    // Prozessdaten dieser Startprozesse laden und entsprechend initialisieren
    
    // Prffall 1
    
    pd1.setZustand( E_ZUSTAND.ANGELEGT );
    pd2.setZustand( E_ZUSTAND.ANGELEGT );
    
    boolean erfuellt = x.isStoppBedingungErfuellt();
    
    if (erfuellt)
      fail ("Prffall 1: Stoppbedingung flschlicherweise erfllt !");

    // Prffall 2
    
    pd1.setZustand( E_ZUSTAND.GESTOPPT );
    pd2.setZustand( E_ZUSTAND.ANGELEGT );
    
    erfuellt = x.isStoppBedingungErfuellt();
    
    if (erfuellt)
      fail ("Prffall 2: Stoppbedingung flschlicherweise erfllt !");

    // Prffall 3
    
    pd1.setZustand( E_ZUSTAND.GESTOPPT );
    pd2.setZustand( E_ZUSTAND.INITIALISIERT );
    
    erfuellt = x.isStoppBedingungErfuellt();
    
    if (erfuellt)
      fail ("Prffall 3: Stoppbedingung flschlicherweise erfllt !");
    
    // Prffall 4
    
    pd1.setZustand( E_ZUSTAND.GESTOPPT );
    pd2.setZustand( E_ZUSTAND.GESTOPPT );
    
    erfuellt = x.isStoppBedingungErfuellt();
    
    if (!erfuellt)
      fail ("Prffall 4: Stoppbedingung flschlicherweise nicht erfllt !");

    // Prffall 5

    pd1.setZustand( E_ZUSTAND.ANGELEGT );
    pd2.setZustand( E_ZUSTAND.ANGELEGT );

    long wartezeit = 10;
    
    prozessDaten.addStoppBedingung   ( prozess1, "", wartezeit); 
    prozessDaten.addStoppBedingung   ( prozess2, "", wartezeit);
    
    Date start = new Date ();
    
    pd1.setZustand( E_ZUSTAND.GESTOPPT );
    pd2.setZustand( E_ZUSTAND.GESTOPPT );

    int t = 0;
    while (t < wartezeit + 10)
    {
      if (x.isStoppBedingungErfuellt())
        break;
      
      mySleep( 1000 );
    }

    erfuellt = false;
    
    if (x.isStoppBedingungErfuellt())
    {
      Date ende = new Date ();
      long delta = (ende.getTime() - start.getTime()) / 1000;
      
      if ((delta < wartezeit) || (delta > wartezeit + 5)) // Kleiner Spielraum nach oben
        fail ("Prffall 5: Stoppbedingung zeitlich nicht korrekt erfllt: " + delta + " Sekunden");
      
      erfuellt = true;
    }
    
    assertEquals( true, erfuellt) ;
  }

  /**
   * Methode initialisiert eine Prozessdatenstruktur mit den oben definierten Testdaten
   */
  public void init ()
  {
    ProzessDaten prozessDaten = new ProzessDaten ();
    ProzessVerwaltung.getInstanz().setProzessDaten( m_name, prozessDaten );

    prozessDaten.setName( m_name );
    prozessDaten.setAusfuehrbareDatei( m_ausfuehrbareDatei );
    
    for (int i=0; i<m_anzAufrufParameter; ++i)
      prozessDaten.addAufrufParameter  ( m_aufrufParameter + " Aufrufparamerer_" + i );
  
    prozessDaten.setStartArt         ( m_option.getText(), m_neustart.getText(), m_intervall );
  
    for (int i=0; i<m_anzVorgaengerBedingungen; ++i)
      prozessDaten.addStartBedingung   ( m_vorgaenger + i, 
                                         m_warteart.getText(), 
                                         m_rechnerVorgaenger    + i, 
                                         m_wartezeitVorgaenger  + i );
  
    for (int i=0; i<m_anzNachfolgerBedingungen; ++i)
      prozessDaten.addStoppBedingung   ( m_nachfolger + i, 
                                         m_rechnerNachfolger    + i, 
                                         m_wartezeitNachfolger  + i );
  
    prozessDaten.setStandardAusgabe       (m_startStoppBlockId, m_standardAusgabe, m_standardDatei);
    prozessDaten.setStandardFehlerAusgabe( m_startStoppBlockId, m_fehlerAusgabe, m_fehlerDatei);

    prozessDaten.setStartVerhaltenFehler( m_startFehlerVerhalten,  m_anzahlStartFehler);
    prozessDaten.setStoppVerhaltenFehler( m_stoppFehlerVerhalten,  m_anzahlStoppFehler); 

    ProzessVerwaltung.getInstanz().setProzessDaten( m_name, prozessDaten );
  }

  /**
   * Sleep Methode
   * @param time Zeit in Millisekunden
   */
  private void mySleep (long time)
  {
    try
    {
      Thread.sleep( time );
    }
    catch (InterruptedException e)
    {
    }
  }

}
