/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.startstopp.prozessvew;

import static org.junit.Assert.*;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Klasse zum Auswerten der JVM Parameter -Xmx, -Xmn und -Xms 
 * @author Dambach Werke GmbH
 *
 */
public class JVMParameterAnalyseTest
{
  private static long _KB = 1024;
  private static long _MB = 1024 * _KB;
  private static long _GB = 1024 * _MB;
  private static long _TB = 1024 * _GB;
  
  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.JVMParameterAnalyse#getXmx()}.
   */
  @Test
  public void testGetXmx()
  {
    HashMap<String, Long> muster = new HashMap<String, Long>();

    muster.put ( "-Xmx=128", (long) (128) );

    muster.put ( "-Xmx=4k", (long) (4 * _KB) );
    muster.put ( "-Xmx=3K", (long) (3 * _KB) );
    
    muster.put ( "-Xmx=1234m", (long) (1234 * _MB) );
    muster.put ( "-Xmx=7654M", (long) (7654 * _MB) );

    muster.put ( "-Xmx=1234g", (long) (1234 * _GB) );
    muster.put ( "-Xmx=7654G", (long) (7654 * _GB) );

    Iterator it = muster.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Entry) it.next();
      
      String parameter = (String) entry.getKey();
      long   value     = (Long) entry.getValue();

      JVMParameterAnalyse jvm = new JVMParameterAnalyse (parameter);
      if (jvm.getXmx() != value)
        fail ("Falscher Rckgabewert bei " + parameter);
    }
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.JVMParameterAnalyse#getXms()}.
   */
  @Test
  public void testGetXms()
  {
    HashMap<String, Long> muster = new HashMap<String, Long>();

    muster.put ( "-Xms=128", (long) (128) );

    muster.put ( "-Xms=4k", (long) (4 * _KB) );
    muster.put ( "-Xms=3K", (long) (3 * _KB) );
    
    muster.put ( "-Xms=1234m", (long) (1234 * _MB) );
    muster.put ( "-Xms=7654M", (long) (7654 * _MB) );

    muster.put ( "-Xms=1234g", (long) (1234 * _GB) );
    muster.put ( "-Xms=7654G", (long) (7654 * _GB) );

    Iterator it = muster.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Entry) it.next();
      
      String parameter = (String) entry.getKey();
      long   value     = (Long) entry.getValue();

      JVMParameterAnalyse jvm = new JVMParameterAnalyse (parameter);
      if (jvm.getXms() != value)
        fail ("Falscher Rckgabewert bei " + parameter);
    }
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.JVMParameterAnalyse#getXmn()}.
   */
  @Test
  public void testGetXmn()
  {
    HashMap<String, Long> muster = new HashMap<String, Long>();

    muster.put ( "-Xmn=128", (long) (128) );

    muster.put ( "-Xmn=4k", (long) (4 * _KB) );
    muster.put ( "-Xmn=3K", (long) (3 * _KB) );
    
    muster.put ( "-Xmn=1234m", (long) (1234 * _MB) );
    muster.put ( "-Xmn=7654M", (long) (7654 * _MB) );

    muster.put ( "-Xmn=1234g", (long) (1234 * _GB) );
    muster.put ( "-Xmn=7654G", (long) (7654 * _GB) );

    Iterator it = muster.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Entry) it.next();
      
      String parameter = (String) entry.getKey();
      long   value     = (Long) entry.getValue();

      JVMParameterAnalyse jvm = new JVMParameterAnalyse (parameter);
      if (jvm.getXmn() != value)
        fail ("Falscher Rckgabewert bei " + parameter);
    }
  }

}
